#include <string.h>
#include <stdlib.h>
#include "wordlist.h"
#include "tokenizer.h"

#define MAX_BUFFER 81

List insertWord(char buffer[], List l)
{
    List newElement = (List)malloc(sizeof(struct ListElementStruct));
    newElement->data.value = createWord(buffer);
    newElement->data.count = 1;
    newElement->next = l;
    return newElement;
}

List addWord(char buffer[], List l)
{
    List resultList = NULL;
    if (l != NULL)
    {
        int compareResult = strcmp(l->data.value, buffer);
        if (compareResult < 0)  //Nessuna corrispondenza --> si continua a cercare
        {
            l->next = addWord(buffer, l->next);
            resultList = l;
        }
        else if (compareResult == 0)    //Trovata corrispondenza
        {
            l->data.count++;
            resultList = l;
        }
        else    //Nessuna corrispondenza --> punto di inserimento
        {
            resultList = insertWord(buffer, l);
        }
    }
    else    //Lista vuota --> inserimento
    {
        resultList = insertWord(buffer, l);
    }

    return resultList;
}

void destroyWordList(List l)
{
    if (l != NULL)
    {
        destroyWordList(l->next);
        destroyWord(l->data.value);
        free(l);
    }
}

RESULT wordCounter(char fileName[], char separatorChars[], List *wordList)
{
    List resultList = NULL;
    RESULT result;
    Tokenizer t;
    char buffer[MAX_BUFFER];

    t = createTokenizer(fileName, separatorChars);

    if (t != NULL)
    {
        BOOL wordGot = getNextToken(t, buffer);
        while (wordGot)
        {
            resultList = addWord(buffer, resultList);
            wordGot = getNextToken(t, buffer);
        }

        destroyTokenizer(t);

        *wordList = resultList;
        result = OK;
    }
    else
    {
        *wordList = NULL;
        result = FILE_ERROR;
    }
    return result;
}

void saveWordList(FILE *f, List l)
{
    if (l != NULL)
    {
        WordCount wc = l->data;
        fprintf(f, "%4d - %s\n", wc.count, wc.value);
        saveWordList(f, l->next);
    }
}